using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using ServerTimers = System.Timers;
using System.Reflection;
using EXT.DOMAIN.vbecs.DAL.VistALink.OpenLibrary;
using EXT.DOMAIN.vbecs.DAL.VistALink.OpenLibrary.Messages;
namespace EXT.DOMAIN.vbecs.DAL.VistALink.Client
/// <summary>
/// Used to notify VistALink consuming classes of connection availability change.
/// </summary>
public delegate void VistALinkConnectionStateChangedDelegate( object sender, VistALinkConnectionStateChangedEventArgs e );
/// <summary>
/// Class internally utilized by <see cref="RpcBroker"/> class to handle network operation.
/// Maintains connection keeping it alive with heartbeats, sends and receives VistALink messages. 
/// </summary>
#if DEBUG || UNIT_TEST
public
#else
internal 
#endif
class VistALinkClientConnection : IDisposable
/// <summary>
/// VistALink connection state. The VistALink connection usually transitions 
/// through all these states during its lifetime.
/// </summary>
protected enum ConnectionState
/// <summary>
/// Connecting
/// </summary>
Connecting,
/// <summary>
/// Connected
/// </summary>
Connected,
/// <summary>
/// Disconnecting
/// </summary>
Disconnecting,
/// <summary>
/// Disconnected
/// </summary>
Disconnected,
/// <summary>
/// Uninitialized
/// </summary>
Uninitialized
private BlockingTimer _heartbeatTimer;
private TcpClient _client;
private NetworkStream _stream;
private ServerConnectionInfo _serverConnectInfo;
private VistALinkMessageXmlFactory _messageFactory;
private VistALinkFaultExceptionFactory _faultXcpFactory;
/// <summary>
/// Flag indicating whether there is a request that was sent, but no response was received yet.
/// </summary>
private bool _requestSent;
private ConnectionState _connectionState = ConnectionState.Uninitialized;
private bool _noStateChangeNotificationFlag;
private bool _serverRespondedToFirstMessageFlag;
/// <summary>
/// Connection state change event.
/// </summary>
public event VistALinkConnectionStateChangedDelegate ConnectionStateChanged;
/// <summary>
/// The only constructor. Initializes the object, and opens a connection to VistA server. 
/// </summary>
/// <param name="serverConnectionInfo">Remote M server connection info.</param>
public VistALinkClientConnection( ServerConnectionInfo serverConnectionInfo )
if( serverConnectionInfo == null )
throw( new ArgumentNullException( "serverConnectionInfo" ) );
_noStateChangeNotificationFlag = false;
_serverConnectInfo = serverConnectionInfo;
_messageFactory = new VistALinkMessageXmlFactory( new RpcParameterXmlFactory() );
_faultXcpFactory = new VistALinkFaultExceptionFactory();
ResetConnectionStateVars();
/// <summary>
/// Connects to VistA server.
/// </summary>
public void Connect()
lock( this )
if( this.IsConnected )
throw( new InvalidOperationException( SR.Exceptions.VistALinkConnectionIsAlreadyOpen() ) );
try
SetConnectionState( ConnectionState.Connecting );
_client = new TcpClient();
_client.SendTimeout = ClientConfig.ReceiveTimeout.ToInt32Milliseconds();
_client.ReceiveTimeout = ClientConfig.SendTimeout.ToInt32Milliseconds();
if (_serverConnectInfo.Domain != null && _serverConnectInfo.Domain != "")
_client.Connect( _serverConnectInfo.Domain, _serverConnectInfo.PortNumber );
else
_client.Connect( _serverConnectInfo.IPAddress, _serverConnectInfo.PortNumber );
_stream = _client.GetStream();
catch( SocketException xcp )
Disconnect( false );
throw( new VistALinkNetworkErrorException( SR.Exceptions.GeneralNetworkError(), xcp ) );
catch( IOException xcp )
Disconnect( false );
// This is required due to 'problematic' implementation of the
// FCL's NetworkStream which catches every exception and wraps it as IOException
if( !(xcp.InnerException is SocketException) )
throw( xcp.InnerException );
throw( new VistALinkNetworkErrorException( SR.Exceptions.GeneralNetworkError(), xcp ) );
catch
Disconnect( false );
throw;
CheckHeartbeat( false );
_requestSent = false;
SetConnectionState( ConnectionState.Connected );
/// <summary>
/// Disconnects from VistA server with or without prior 
/// notification (depending on parameter specified).
/// </summary>
/// <param name="notifyServer">
///
Flag indicating whether server should be notified before disconnect.
///
Using 'false' value will immediately drop communication with the server.
///
</param>
private void Disconnect( bool notifyServer )
lock( this )
try
try
bool _wasConnected = IsConnected;
SetConnectionState( ConnectionState.Disconnecting );
if( _heartbeatTimer != null )
_heartbeatTimer.Close();
if( !notifyServer || !_wasConnected )
return;
NotifyServerAboutDisconnect();
finally
if( _client != null )
_client.Close();
if( _stream != null )
_stream.Close();
finally
ResetConnectionStateVars();
private void NotifyServerAboutDisconnect()
bool _nonEmtpyBufferFlag = _requestSent;
// Need to reset flag here because disconnect may occur at any point
_requestSent = false;
UntimedSendMessage( new VistALinkCloseSocketRequestMessage() );
// if there were no message pending in receive buffer (no request sent to server), 
// we can safely receive server response
if( !_nonEmtpyBufferFlag )
VistALinkCloseSocketResponseMessage _response = (VistALinkCloseSocketResponseMessage)UntimedReceiveMessage();
if( _response.ResponseStatus != ResponseStatus.Success )
throw( new VistALinkException( SR.Exceptions.ServerFailureWhileClosingConnection() ) );
/// <summary>
/// Sends specified VistALink message to the server 
/// converting message to byte array and writing to the network stream 
/// without starting/stopping heartbeat.
/// </summary>
/// <param name="messageToSend">Client message to send to remote server.</param>
private void UntimedSendMessage( VistALinkMessage messageToSend )
try
if( messageToSend == null )
throw( new ArgumentNullException( "messageToSend" ) );
if( _requestSent )
throw( new InvalidOperationException( SR.Exceptions.PrematureWriteOccured() ) );
byte[] _dataChunk = messageToSend.ToXmlByteArray();
_stream.Write( _dataChunk, 0, _dataChunk.Length );
_stream.WriteByte( GlobalConfig.EOTByte );
_requestSent = true;
catch( IOException xcp )
Disconnect( false );
// This is required due to 'problematic' implementation of the
// FCL's NetworkStream.Read / Write which catches every exception and wraps it as IOException
if( !(xcp.InnerException is SocketException) )
throw( xcp.InnerException );
throw( new VistALinkNetworkErrorException( SR.Exceptions.GeneralNetworkError(), xcp ) );
catch
Disconnect( false );
throw;
/// <summary>
/// Receives message reading data from the network stream, parsing to XML and creating 
/// message without starting/stopping heartbeat.
/// </summary>
/// <returns>Received VistALink message of appropriate type.</returns>
private VistALinkMessage UntimedReceiveMessage()
return ParseIncomingXmlCreateMessage( ReceiveIncomingMessageXml() );
/// <summary>
/// Parses XML received from VistA server and returns appropriate <see cref="VistALinkMessage"/> object 
/// or throws an exception if fault message is received.
/// </summary>
/// <param name="incomingMessageXml">XML string received from VistA server.</param>
/// <returns><see cref="VistALinkMessage"/> object representing message received from remote server.</returns>
private VistALinkMessage ParseIncomingXmlCreateMessage( string incomingMessageXml )
if( incomingMessageXml == null )
throw( new ArgumentNullException( "incomingMessageXml" ) );
try
VistALinkMessage _msg = _messageFactory.ParseXmlCreateVistALinkMessage( incomingMessageXml );
if( _msg == null ) 
throw( new VistALinkException( SR.Exceptions.UnknownNetworkMessageReceived() ) );
if( !(_msg is BaseFaultMessage) )
return _msg;
throw( _faultXcpFactory.CreateVistALinkExceptionFromFaultMessage( (BaseFaultMessage)_msg ) );
catch( VistALinkFaultException xcp )
if( xcp.IsFatal )
Disconnect( false );
throw;
catch
Disconnect( true );
throw;
/// <summary>
/// Receives incoming XML string from VistA server.
/// </summary>
/// <returns>String, received from VistA server.</returns>
private string ReceiveIncomingMessageXml()
MemoryStream _ms;
try
if( !_requestSent )
throw( new InvalidOperationException( SR.Exceptions.PrematureReadOccured() ) );
_ms = new MemoryStream( ClientConfig.ReceiveBufferSize );
byte[] _buf = new byte[ ClientConfig.ReceiveBufferSize ];
for(;;)
int _bytesRead = _stream.Read( _buf, 0, _buf.Length );
if( _bytesRead <= 0 )
if( _serverRespondedToFirstMessageFlag )
throw( new VistALinkException( SR.Exceptions.UnexpectedEOTOccured( Convert.ToInt32( GlobalConfig.EOTByte ) ) ) );
else
throw( new VistALinkNetworkErrorException( SR.Exceptions.VistAServerActivelyRefusedVistALinkConnection() ) );
if( _buf[_bytesRead - 1] == GlobalConfig.EOTByte )
_ms.Write( _buf, 0, _bytesRead - 1 );
break;
else
_ms.Write( _buf, 0, _bytesRead );
_requestSent = false;
if( !_serverRespondedToFirstMessageFlag )
_serverRespondedToFirstMessageFlag = true;
return GlobalConfig.NetworkMessageEncoding.GetString( _ms.ToArray() );
catch( IOException xcp )
Disconnect( false );
// This is required due to 'problematic' implementation of the
// FCL's NetworkStream.Read / Write which catches every exception and wraps it as IOException
if( !(xcp.InnerException is SocketException) )
throw( xcp.InnerException );
throw( new VistALinkNetworkErrorException( SR.Exceptions.GeneralNetworkError(), xcp ) );
catch
Disconnect( false );
throw;
/// <summary>
/// Checks VistALink availability by sending heartbeat request, receiving a heartbeat response
/// and adjusting timer interval to new heartbeat rate. Also automatically schedules a timer 
/// callback to keep connection alive
/// </summary>
/// <returns>True if connection is available. Otherwise - false.</returns>
public bool CheckHeartbeatGetStatus()
lock( this )
if( !IsConnected )
return false;
CheckHeartbeat();
return IsConnected;
/// <summary>
/// Default wrapper for <see cref="CheckHeartbeat"/> method suppressing exceptions caused by 
/// VistA server unavailability. Works as heartbeat timer callback method.
/// </summary>
private void CheckHeartbeat()
CheckHeartbeat( true );
/// <summary>
/// Checks VistALink availability by sending heartbeat request, receiving a heartbeat response
/// and adjusting timer interval to new heartbeat rate. Also automatically schedules a timer 
/// callback to keep connection alive
/// </summary>
/// <param name="suppressUnavailabilityException">
///
Parameter indicating whether exceptions caused by network problems / VistA server unavailability 
///
should be suppressed. Used when establishing a connection - exceptions should not be masked at that point.
///
</param>
private void CheckHeartbeat( bool suppressUnavailabilityException )
lock( this )
try
UntimedSendMessage( new VistALinkHeartbeatRequestMessage() );
VistALinkHeartbeatResponseMessage _msg = (VistALinkHeartbeatResponseMessage)UntimedReceiveMessage();
if( _heartbeatTimer == null )
_heartbeatTimer = new BlockingTimer( _msg.HeartbeatRate, true, new BlockingTimerCallback( CheckHeartbeat ) );
else
_heartbeatTimer.Interval = _msg.HeartbeatRate;
if( !_heartbeatTimer.IsEnabled )
_heartbeatTimer.Start();
catch( VistALinkNetworkErrorException ) 
Disconnect( false );
if( !suppressUnavailabilityException ) // Suppresing an exception if VistALink is simply "down"
throw;
catch // If the problem is not in network - raise a red flag.
Disconnect( true );
throw;
/// <summary>
/// Sends VistALink message to server and receives response message. 
/// </summary>
/// <param name="clientMessage">Message to send.</param>
/// <returns>Server response message.</returns>
public VistALinkMessage SendReceiveMessage( VistALinkMessage clientMessage )
lock( this )
if( !IsConnected )
throw( new InvalidOperationException( SR.Exceptions.VistALinkConnectionIsNotYetOpen() ) );
try
_heartbeatTimer.Stop();
UntimedSendMessage( clientMessage );
VistALinkMessage _result = UntimedReceiveMessage();
_heartbeatTimer.Start();
if( clientMessage is VistALinkCloseSocketRequestMessage )
Disconnect( false );
return _result;
catch
Disconnect( false );
throw;
/// <summary>
/// Finalization method closing the connection (disconnecting from server). 
/// </summary>
~VistALinkClientConnection()
Dispose( false );
/// <summary>
/// Cleanup method needed to implement <see cref="IDisposable"/>.
/// </summary>
public void Dispose()
GC.SuppressFinalize( this );
Dispose( true );
/// <summary>
/// Closes the connection.
/// </summary>
public void Close()
Dispose();
/// <summary>
/// Standard implementation of the dispose method. 
/// </summary>
/// <param name="disposing">
///
Flag indicating if the disposition was invoked explicitly under normal 
///
conditions (true) or forced upon object disposal (false).
///
</param>
private void Dispose( bool disposing )
lock( this )
if( disposing )
Disconnect( true );
ResetConnectionStateVars();
_serverConnectInfo = null;
/// <summary>
/// Resets connection state and network related member variables to default disconnected state.
/// </summary>
private void ResetConnectionStateVars()
_client = null;
_stream = null;
_heartbeatTimer = null;
_requestSent = false;
_serverRespondedToFirstMessageFlag = false;
SetConnectionState( ConnectionState.Disconnected );
/// <summary>
/// Remote server connection information.
/// </summary>
public ServerConnectionInfo ServerConnectionInfo
get
return _serverConnectInfo;
/// <summary>
/// Indicates whether connection with VistA server is established. 
/// </summary>
public bool IsConnected
get
lock( this )
return IsConnectedState( _connectionState );
/// <summary>
/// Determines whether a given connection state is available ('connected') state - 
/// state in which connection may be used by outside callers.
/// </summary>
/// <param name="state"></param>
/// <returns></returns>
private bool IsConnectedState( ConnectionState state )
return state == ConnectionState.Connected;
/// <summary>
/// Sets connection state and notifies registered handlers if state change has occured.
/// </summary>
/// <param name="connectionStateToSet">New connection state to set.</param>
private void SetConnectionState( ConnectionState connectionStateToSet )
lock( this )
ConnectionState _previousConnectionState = _connectionState;
_connectionState = connectionStateToSet;
if( _noStateChangeNotificationFlag )
return;
bool _isNewStateConnected = IsConnectedState( connectionStateToSet );
try
if( IsConnectedState( _previousConnectionState ) != _isNewStateConnected )
OnConnectionStateChanged( new VistALinkConnectionStateChangedEventArgs( _isNewStateConnected ) );
catch
// Error occured in consumer connection availability event handler.
// Triggering it again may result in an infinite loop.
_noStateChangeNotificationFlag = true;
Close();
throw;
/// <summary>
/// Connection state change notification method.
/// </summary>
/// <param name="eventArgs">Connection state change event args.</param>
protected virtual void OnConnectionStateChanged( VistALinkConnectionStateChangedEventArgs eventArgs )
if( ConnectionStateChanged != null )
ConnectionStateChanged.BeginInvoke( this, eventArgs, new AsyncCallback( ConnectionStateChangedEndInvokeHandler ), null );
/// <summary>
/// Finalizer for async connection state change notification method.
/// </summary>
/// <param name="ar">Async result. Not used.</param>
private void ConnectionStateChangedEndInvokeHandler( IAsyncResult ar )
if( ConnectionStateChanged != null )
ConnectionStateChanged.EndInvoke( ar );
