using System;
using System.Xml;
using EXT.DOMAIN.vbecs.DAL.VistALink.OpenLibrary;
namespace EXT.DOMAIN.vbecs.DAL.VistALink.Listener.Core
/// <summary>
/// This class represents security information associated with published 
/// procedure on RPC server. In essence, it's a security token used to authorize 
/// caller requesting execution of published procedure. 
/// 
/// The class deserializes itself from host application's XML configuration file 
/// and provides all means to authorize calling application. 
/// </summary>
public class PublishedProcedureSecurityInfo
// Constants used in XML message serialization/deserialization
private const string XMLCONSTS_SECURITY_INFO_NODE_NAME = "SecurityInfo";
private const string XMLCONSTS_SECURITY_CONTEXT_NODE_NAME = "SecurityContext";
private string _securityContext;
private PublishedProcedureIpAccessList _ipAccessList;
/// <summary>
/// Constructor accepting all information required to create class as set of parameters. 
/// May be used by derived classes for extendability purposes. 
/// </summary>
/// <param name="securityContext">
///
Published procedure security context. 
///
Basically, it's an arbitrary string that caller should provide to run published procedure.
///
</param>
/// <param name="ipAccessList">
///
List of IP addresses that are 
///
granted access for published procedure on RPC server.
///
</param>
protected PublishedProcedureSecurityInfo( string securityContext, PublishedProcedureIpAccessList ipAccessList )
if( securityContext == null )
throw( new ArgumentNullException( "securityContext" ) );
if( ipAccessList == null )
throw( new ArgumentNullException( "ipAccessList" ) );
_securityContext = securityContext;
_ipAccessList = ipAccessList;
/// <summary>
/// XML deserialization factory method creating new instance 
/// of the class from supplied XML node.
/// </summary>
/// <param name="sourceNode">Source XML node to parse and deserialize object from.</param>
/// <returns>Instance of the class deserialized from given XML node.</returns>
public static PublishedProcedureSecurityInfo Parse( XmlNode sourceNode )
if( sourceNode == null )
throw( new ArgumentNullException( "sourceNode" ) );
XmlUtility.ParseValidateConvertNodeToElement( sourceNode, XMLCONSTS_SECURITY_INFO_NODE_NAME );
XmlNodeList _childElements = XmlUtility.ParseGetChildElements( sourceNode );
if( _childElements.Count != 2 )
throw( new XmlParseException( SR.Exceptions.PPSecurityInfoNodeMustHave2Elements( XMLCONSTS_SECURITY_INFO_NODE_NAME ) ) );
XmlUtility.ParseValidateConvertNodeToElement( _childElements[0], XMLCONSTS_SECURITY_CONTEXT_NODE_NAME );
return new PublishedProcedureSecurityInfo( 
_childElements[0].InnerText, 
PublishedProcedureIpAccessList.Parse( _childElements[1] ) );
/// <summary>
/// Verfies security evidence supplied by caller. It will throw exception if 
/// caller is not unauthorized to run published procedure. 
/// Derived classes should extend this method to implement additional verification.
/// </summary>
/// <param name="evidence">Security evidence token supplied by caller.</param>
public virtual void Verify( CallerSecurityEvidence evidence )
if( evidence == null )
throw( new ArgumentNullException( "evidence" ) );
if( !_ipAccessList.IsIpAllowedAccess( evidence.ClientIP ) )
throw( new CallerNotAuthorizedSecurityException( SR.Exceptions.PPSecurityInfoVerificationFailedIpIsNotAllowed() ) );
if( evidence.SecurityContext != _securityContext )
throw( new CallerNotAuthorizedSecurityException( SR.Exceptions.PPSecurityInfoVerificationFailedSecurityContextMismatch() ) );
