using System;
using System.Xml;
using System.Text;
using System.Configuration;
using System.Collections.Specialized;
namespace EXT.DOMAIN.vbecs.DAL.VistALink.OpenLibrary
/// <summary>
/// <see cref="GlobalConfig"/> class provides configuration settings for the 
/// VistALink Open Library. It automatically detects if VistALink section exists 
/// in the calling application configuration file, if yes - it takes all 
/// available settings from the latter. If not - default hardcoded values are
/// used. The class, however, does not provide a standard way to modify its
/// properties programmatically. 
/// </summary>
public class GlobalConfig
// Consts used to read settings from config file
private const string XMLCONSTS_CONFIG_ROOT_SECTION_NAME = "VistALink";
private const string XMLCONSTS_CONFIG_COMMON_SECTION_NAME = "Common";
private const string CONFIG_VALUE_MAX_PARAMETERS_NUMBER = "MaxRpcParametersNumber";
private const string CONFIG_VALUE_NETWORK_MESSAGE_ENCODING = "NetworkMessageEncoding";
private const string CONFIG_VALUE_RPC_XML_NULL_VALUE = "RpcXmlNullValue";
private const string CONFIG_VALUE_EOT_BYTE_CODE = "EotByteCode";
private const string CONFIG_VALUE_DEFAULT_RPC_REQUEST_TIMEOUT = "DefaultRpcTimeout";
// Variables containing configured values
private static Encoding _networkMessageEncoding;
private static int _maxRpcParametersNumber;
private static string _rpcXmlNullValue;
private static byte _eotByte;
private static Timeout _defaultRpcRequestTimeout;
private static ServerConnectionInfoCollection _serverList;
/// <summary>
/// Class should not be instantiated.
/// </summary>
private GlobalConfig() {}
/// <summary>
/// Initializes <see cref="GlobalConfig"/> on first request.
/// </summary>
static GlobalConfig()
SetDefaultHardcodedValues();
GetSettingsFromConfigFile();
GetServerListFromConfigFile();
/// <summary>
/// Defines encoding for VistALink protocol network messages
/// </summary>
public static Encoding NetworkMessageEncoding
get
return _networkMessageEncoding;
/// <summary>
/// Maximum number of RPC request parameters 
/// specified in VistALink protocol.
/// </summary>
public static int MaxRpcParametersNumber 
get
return _maxRpcParametersNumber;
/// <summary>
/// This value is used to substitute null values when forming 
/// RPC request to M server. 
/// </summary>
public static string RpcXmlNullValue
get
return _rpcXmlNullValue;
/// <summary>
/// End-of-transmission (EOT) character used to separate network messages.
/// When defined in config file, integer code must be used. 
/// </summary>
public static byte EOTByte
get
return _eotByte;
/// <summary>
/// Default RPC timeout. 
/// </summary>
public static Timeout DefaultRpcRequestTimeout
get
return _defaultRpcRequestTimeout;
/// <summary>
/// Sets default hardcoded values. 
/// </summary>
private static void SetDefaultHardcodedValues()
_networkMessageEncoding = Encoding.ASCII;
_maxRpcParametersNumber = 10;
_rpcXmlNullValue = String.Empty;
_eotByte = Convert.ToByte( 4 );
_defaultRpcRequestTimeout = new Timeout( 30000 );
_serverList = null;
/// <summary>
/// Retrieves settings from application configuration file if they are available.
/// Overrides defaults for defined values only.
/// </summary>
private static void GetSettingsFromConfigFile()
NameValueCollection _configFileSettings = (NameValueCollection)ConfigurationSettings.GetConfig( GetXPathToChildConfigSection( XMLCONSTS_CONFIG_COMMON_SECTION_NAME ) );
// Check if config section exists
if( _configFileSettings == null ) 
return;
if( _configFileSettings[ CONFIG_VALUE_MAX_PARAMETERS_NUMBER ] != null )
_maxRpcParametersNumber = Int32.Parse( (string)_configFileSettings[ CONFIG_VALUE_MAX_PARAMETERS_NUMBER ] );
if( _configFileSettings[ CONFIG_VALUE_RPC_XML_NULL_VALUE ] != null )
_rpcXmlNullValue = (string)_configFileSettings[ CONFIG_VALUE_RPC_XML_NULL_VALUE ];
if( _configFileSettings[ CONFIG_VALUE_EOT_BYTE_CODE ] != null )
_eotByte = Convert.ToByte( Int32.Parse( (string)_configFileSettings[ CONFIG_VALUE_EOT_BYTE_CODE ] ) );
if( _configFileSettings[ CONFIG_VALUE_DEFAULT_RPC_REQUEST_TIMEOUT ] != null )
_defaultRpcRequestTimeout = Timeout.ParseStringInSeconds( (string)_configFileSettings[ CONFIG_VALUE_DEFAULT_RPC_REQUEST_TIMEOUT ] );
if( _configFileSettings[ CONFIG_VALUE_NETWORK_MESSAGE_ENCODING ] != null )
_networkMessageEncoding = Encoding.GetEncoding( (string)_configFileSettings[ CONFIG_VALUE_NETWORK_MESSAGE_ENCODING ] );
/// <summary>
/// Retrieves list of servers from application configuration file.
/// </summary>
private static void GetServerListFromConfigFile()
_serverList = 
(ServerConnectionInfoCollection)ConfigurationSettings.GetConfig( 
GetXPathToChildConfigSection( ServerListConfigSectionHandler.SectionRootElementName ) );
/// <summary>
/// The method provides a standard way to store and access M server connection
/// information in application configuration file. It will return server info,
/// if the record for the server with given handle is available, and null otherwise. 
/// Please, understand, that it will only work when server information
/// is available in mapped configuration file. 
/// </summary>
/// <param name="serverHandle">
///
Internal handle used to indentify M server in configuration file
///
</param>
/// <returns>
///
Instance of <see cref="ServerConnectionInfo"/> if server information is found. 
///
Null otherwise.
///
</returns>
public static ServerConnectionInfo GetServerConnectionInfoByHandle( string serverHandle )
if( serverHandle == null )
throw( new ArgumentNullException( "serverHandle" ) );
if( _serverList == null ) 
return null;
return _serverList.GetServerConnectionInfoByHandle( serverHandle );
/// <summary>
/// Forms string XPath to a section with a given name that 
/// is a child of VistALink application configuration section.
/// </summary>
/// <param name="childSectionName">Name of child section to form XPath to.</param>
/// <returns>
///
String XPath to a given child section of 
///
VistALink application configuration section.
///
</returns>
public static string GetXPathToChildConfigSection( string childSectionName )
return String.Format( "{0}/{1}", XMLCONSTS_CONFIG_ROOT_SECTION_NAME, childSectionName ); 
